@@ -8,8 +8,8 @@
 #include "../common/control.h"
 #include "../common/orientation_sensor.h"
 #include "config_handler.h"
+#include "globalscalesettings.h"
 #include "kcm_screen_debug.h"
-#include "output_identifier.h"
 #include "output_model.h"
 
 #include <kscreen/config.h>
@@ -24,6 +24,9 @@
 #include <KPluginFactory>
 #include <KSharedConfig>
 
+#include <QDBusConnection>
+#include <QDBusMessage>
+#include <QDBusPendingReply>
 #include <QTimer>
 
 K_PLUGIN_FACTORY_WITH_JSON(KCMDisplayConfigurationFactory, "kcm_kscreen.json", registerPlugin<KCMKScreen>();)
@@ -31,9 +34,9 @@ K_PLUGIN_FACTORY_WITH_JSON(KCMDisplayConfigurationFactory, "kcm_kscreen.json", r
 using namespace KScreen;
 
 KCMKScreen::KCMKScreen(QObject *parent, const KPluginMetaData &data, const QVariantList &args)
-    : KQuickAddons::ConfigModule(parent, data, args)
+    : KQuickAddons::ManagedConfigModule(parent, data, args)
 {
-    qmlRegisterType<OutputModel>();
+    qmlRegisterAnonymousType<OutputModel>("org.kde.private.kcm.screen", 1);
     qmlRegisterType<KScreen::Output>("org.kde.private.kcm.kscreen", 1, 0, "Output");
     qmlRegisterUncreatableType<Control>("org.kde.private.kcm.kscreen", 1, 0, "Control", QStringLiteral("Provides only the OutputRetention enum class"));
     Log::instance();
@@ -49,6 +52,9 @@ KCMKScreen::KCMKScreen(QObject *parent, const KPluginMetaData &data, const QVari
     connect(m_orientationSensor, &OrientationSensor::availableChanged, this, &KCMKScreen::orientationSensorAvailableChanged);
 
     connect(KScreen::ConfigMonitor::instance(), &KScreen::ConfigMonitor::configurationChanged, this, &KCMKScreen::updateFromBackend);
+
+    registerSettings(GlobalScaleSettings::self());
+    connect(GlobalScaleSettings::self(), &GlobalScaleSettings::scaleFactorChanged, this, &KCMKScreen::globalScaleChanged);
 }
 
 void KCMKScreen::configReady(ConfigOperation *op)
@@ -56,6 +62,8 @@ void KCMKScreen::configReady(ConfigOperation *op)
     qCDebug(KSCREEN_KCM) << "Reading in config now.";
     if (op->hasError()) {
         m_configHandler.reset();
+        m_configNeedsSave = false;
+        settingsChanged();
         Q_EMIT backendError();
         return;
     }
@@ -67,6 +75,7 @@ void KCMKScreen::configReady(ConfigOperation *op)
     m_configHandler->setConfig(config);
     setBackendReady(true);
     Q_EMIT perOutputScalingChanged();
+    Q_EMIT xwaylandClientsScaleSupportedChanged();
     Q_EMIT primaryOutputSupportedChanged();
     Q_EMIT outputReplicationSupportedChanged();
     Q_EMIT tabletModeAvailableChanged();
@@ -86,7 +95,6 @@ void KCMKScreen::save()
 void KCMKScreen::revertSettings()
 {
     if (!m_configHandler) {
-        setNeedsSave(false);
         return;
     }
     if (!m_settingsReverted) {
@@ -99,6 +107,15 @@ void KCMKScreen::revertSettings()
     }
 }
 
+void KCMKScreen::requestReboot()
+{
+    QDBusMessage msg = QDBusMessage::createMethodCall(QStringLiteral("org.kde.LogoutPrompt"),
+                                                      QStringLiteral("/LogoutPrompt"),
+                                                      QStringLiteral("org.kde.LogoutPrompt"),
+                                                      QStringLiteral("promptReboot"));
+    QDBusConnection::sessionBus().asyncCall(msg);
+}
+
 void KCMKScreen::setStopUpdatesFromBackend(bool value)
 {
     m_stopUpdatesFromBackend = value;
@@ -153,9 +170,12 @@ void KCMKScreen::doSave(bool force)
         return;
     }
 
-    if (!perOutputScaling()) {
-        writeGlobalScale();
+    const bool globalScaleChanged = GlobalScaleSettings::self()->isSaveNeeded();
+    ManagedConfigModule::save();
+    if (globalScaleChanged) {
+        exportGlobalScale();
     }
+
     m_configHandler->writeControl();
 
     // Store the current config, apply settings. Block until operation is
@@ -167,7 +187,6 @@ void KCMKScreen::doSave(bool force)
 
     const auto updateInitialData = [this]() {
         if (!m_configHandler) {
-            setNeedsSave(false);
             return;
         }
         m_configHandler->updateInitialData();
@@ -213,13 +232,11 @@ OutputModel *KCMKScreen::outputModel() const
 
 void KCMKScreen::identifyOutputs()
 {
-    if (!m_configHandler || !m_configHandler->initialConfig() || m_outputIdentifier) {
-        return;
-    }
-    m_outputIdentifier.reset(new OutputIdentifier(m_configHandler->initialConfig(), this));
-    connect(m_outputIdentifier.get(), &OutputIdentifier::identifiersFinished, this, [this]() {
-        m_outputIdentifier.reset();
-    });
+    const QString name = QStringLiteral("org.kde.KWin");
+    const QString interface = QStringLiteral("org.kde.KWin.Effect.OutputLocator1");
+    const QString path = QStringLiteral("/org/kde/KWin/Effect/OutputLocator1");
+    auto message = QDBusMessage::createMethodCall(name, path, interface, QStringLiteral("show"));
+    QDBusConnection::sessionBus().send(message);
 }
 
 QSize KCMKScreen::normalizeScreen() const
@@ -299,12 +316,14 @@ void KCMKScreen::load()
 {
     qCDebug(KSCREEN_KCM) << "About to read in config.";
 
+    ManagedConfigModule::load();
+
     setBackendReady(false);
-    setNeedsSave(false);
+    m_configNeedsSave = false;
+    settingsChanged();
     if (!screenNormalized()) {
         Q_EMIT screenNormalizedChanged();
     }
-    fetchGlobalScale();
 
     // Don't pull away the outputModel under QML's feet
     // signal its disappearance first before deleting and replacing it.
@@ -318,6 +337,7 @@ void KCMKScreen::load()
 
     m_configHandler.reset(new ConfigHandler(this));
     Q_EMIT perOutputScalingChanged();
+    Q_EMIT xwaylandClientsScaleSupportedChanged();
     connect(m_configHandler.get(), &ConfigHandler::outputModelChanged, this, &KCMKScreen::outputModelChanged);
     connect(m_configHandler.get(), &ConfigHandler::outputConnect, this, [this](bool connected) {
         Q_EMIT outputConnect(connected);
@@ -342,29 +362,17 @@ void KCMKScreen::load()
 
 void KCMKScreen::continueNeedsSaveCheck(bool needs)
 {
-    if (needs || m_globalScale != m_initialGlobalScale) {
-        setNeedsSave(true);
-    } else {
-        setNeedsSave(false);
-    }
+    m_configNeedsSave = needs;
+    settingsChanged();
 }
 
-void KCMKScreen::fetchGlobalScale()
+bool KCMKScreen::isSaveNeeded() const
 {
-    const auto config = KSharedConfig::openConfig(QStringLiteral("kdeglobals"));
-    const qreal scale = config->group("KScreen").readEntry("ScaleFactor", 1.0);
-    m_initialGlobalScale = scale;
-    setGlobalScale(scale);
+    return m_configNeedsSave;
 }
 
-void KCMKScreen::writeGlobalScale()
+void KCMKScreen::exportGlobalScale()
 {
-    if (qFuzzyCompare(m_initialGlobalScale, m_globalScale)) {
-        return;
-    }
-    auto config = KSharedConfig::openConfig(QStringLiteral("kdeglobals"));
-    config->group("KScreen").writeEntry("ScaleFactor", m_globalScale);
-
     // Write env var to be used by session startup scripts to populate the QT_SCREEN_SCALE_FACTORS
     // env var.
     // We use QT_SCREEN_SCALE_FACTORS as opposed to QT_SCALE_FACTOR as we need to use one that will
@@ -374,14 +382,15 @@ void KCMKScreen::writeGlobalScale()
     QString screenFactors;
     const auto outputs = m_configHandler->config()->outputs();
     for (const auto &output : outputs) {
-        screenFactors.append(output->name() + QLatin1Char('=') + QString::number(m_globalScale) + QLatin1Char(';'));
+        screenFactors.append(output->name() + QLatin1Char('=') + QString::number(globalScale()) + QLatin1Char(';'));
     }
+    auto config = KSharedConfig::openConfig("kdeglobals");
     config->group("KScreen").writeEntry("ScreenScaleFactors", screenFactors);
 
     KConfig fontConfig(QStringLiteral("kcmfonts"));
     auto fontConfigGroup = fontConfig.group("General");
 
-    if (qFuzzyCompare(m_globalScale, 1.0)) {
+    if (qFuzzyCompare(globalScale(), 1.0)) {
         // if dpi is the default (96) remove the entry rather than setting it
         QProcess queryProc;
         queryProc.start(QStringLiteral("xrdb"), {QStringLiteral("-query")});
@@ -411,39 +420,48 @@ void KCMKScreen::writeGlobalScale()
         }
         fontConfigGroup.writeEntry("forceFontDPI", 0);
     } else {
-        const int scaleDpi = qRound(m_globalScale * 96.0);
+        const int scaleDpi = qRound(globalScale() * 96.0);
         QProcess proc;
         proc.start(QStringLiteral("xrdb"), {QStringLiteral("-quiet"), QStringLiteral("-merge"), QStringLiteral("-nocpp")});
         if (proc.waitForStarted()) {
-            proc.write(QByteArray("Xft.dpi: " + QString::number(scaleDpi).toLatin1()));
+            proc.write(QByteArray("Xft.dpi: ") + QByteArray::number(scaleDpi));
             proc.closeWriteChannel();
             proc.waitForFinished();
         }
         fontConfigGroup.writeEntry("forceFontDPI", scaleDpi);
     }
 
-    m_initialGlobalScale = m_globalScale;
     Q_EMIT globalScaleWritten();
 }
 
 qreal KCMKScreen::globalScale() const
 {
-    return m_globalScale;
+    return GlobalScaleSettings::self()->scaleFactor();
 }
 
 void KCMKScreen::setGlobalScale(qreal scale)
 {
-    if (qFuzzyCompare(m_globalScale, scale)) {
-        return;
-    }
-    m_globalScale = scale;
-    if (m_configHandler) {
-        m_configHandler->checkNeedsSave();
-    } else {
-        continueNeedsSaveCheck(false);
-    }
+    GlobalScaleSettings::self()->setScaleFactor(scale);
     Q_EMIT changed();
-    Q_EMIT globalScaleChanged();
+}
+
+bool KCMKScreen::xwaylandClientsScale() const
+{
+    return GlobalScaleSettings::self()->xwaylandClientsScale();
+}
+
+void KCMKScreen::setXwaylandClientsScale(bool scale)
+{
+    GlobalScaleSettings::self()->setXwaylandClientsScale(scale);
+    Q_EMIT changed();
+}
+
+bool KCMKScreen::xwaylandClientsScaleSupported() const
+{
+    if (!m_configHandler || !m_configHandler->config()) {
+        return false;
+    }
+    return m_configHandler->config()->supportedFeatures().testFlag(Config::Feature::XwaylandScales);
 }
 
 int KCMKScreen::outputRetention() const
