@@ -164,10 +164,19 @@ KScreen::ConfigPtr Generator::displaySwitch(DisplaySwitchAction action)
     KScreen::OutputPtr embedded, external;
     embedded = embeddedOutput(connectedOutputs);
     // If we don't have an embedded output (desktop with two external screens
-    // for instance), then pretend one of them is embedded
+    // for instance), then pretend the current primary one is embedded
     if (!embedded) {
-        const auto &firstConnectedOutputKey = connectedOutputs.constBegin().key();
-        embedded = connectedOutputs.value(firstConnectedOutputKey);
+        // Find primary screen
+        for (auto &screen : connectedOutputs) {
+            if (screen->isPrimary()) {
+                embedded = screen;
+                break;
+            }
+        }
+        if (!embedded) {
+            // If all else fail take the first screen
+            embedded = connectedOutputs.constBegin().value();
+        }
     }
     // Just to be sure
     if (embedded->modes().isEmpty()) {
@@ -182,8 +191,7 @@ KScreen::ConfigPtr Generator::displaySwitch(DisplaySwitchAction action)
     }
 
     connectedOutputs.remove(embedded->id());
-    const auto firstConnectedOutputKey = connectedOutputs.constBegin().key();
-    external = connectedOutputs.value(firstConnectedOutputKey);
+    external = connectedOutputs.constBegin().value();
     // Just to be sure
     if (external->modes().isEmpty()) {
         return config;
@@ -192,6 +200,20 @@ KScreen::ConfigPtr Generator::displaySwitch(DisplaySwitchAction action)
     Q_ASSERT(embedded->currentMode());
     Q_ASSERT(external->currentMode());
 
+    // Change action to be relative to embedded screen
+    if (!embedded->isPrimary()) {
+        switch (action) {
+        case Generator::ExtendToLeft:
+            action = Generator::ExtendToRight;
+            break;
+        case Generator::ExtendToRight:
+            action = Generator::ExtendToLeft;
+            break;
+        default:
+            break;
+        }
+    }
+
     switch (action) {
     case Generator::ExtendToLeft: {
         qCDebug(KSCREEN_KDED) << "Extend to left";
@@ -201,7 +223,6 @@ KScreen::ConfigPtr Generator::displaySwitch(DisplaySwitchAction action)
         const QSize size = external->geometry().size();
         embedded->setPos(QPoint(size.width(), 0));
         embedded->setEnabled(true);
-        embedded->setPrimary(true);
 
         return config;
     }
@@ -228,13 +249,11 @@ KScreen::ConfigPtr Generator::displaySwitch(DisplaySwitchAction action)
         qCDebug(KSCREEN_KDED) << "Extend to the right";
         embedded->setPos(QPoint(0, 0));
         embedded->setEnabled(true);
-        embedded->setPrimary(true);
 
         Q_ASSERT(embedded->currentMode()); // we must have a mode now
         const QSize size = embedded->geometry().size();
         external->setPos(QPoint(size.width(), 0));
         external->setEnabled(true);
-        external->setPrimary(false);
 
         return config;
     }
